//
//  NSData_Base64Encoding.h
//
//  Created by Ben Spratling & Graham Booker on 8/3/08.
//	Actually it was mostly Graham
//  Copyright 2008 Ben & Graham. All rights reserved.
//	You may use this code in any program for any reason, and please, please ask Apple to include this in Core Foundation
//	When they do, ask them to add in error checking...

#import "NSData_Base64Encoding.h"

@implementation NSData (Base64Encoding)
- (NSString *)stringWithBase64Encoding
{
	const unsigned char *input = [self bytes];
	NSUInteger inLength = [self length];
	NSUInteger outLength;
	NSUInteger mod;
	outLength = inLength;
	if ((mod = outLength % 3) != 0){
		outLength = outLength + 3 - mod;
	}
	outLength = outLength * 4 / 3;
	char output[outLength];
	char encodeLookup[64] = {
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
		'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
		'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
		'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
		'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
		'w', 'x', 'y', 'z', '0', '1', '2', '3',
		'4', '5', '6', '7', '8', '9', '+', '/'
	};

	char *outBuffer=output;
	const unsigned char *inBuffer=input;
	int inLengthDiv = inLength - (inLength % 3);
	while(inBuffer - input < inLengthDiv)
	{
		outBuffer[0]=encodeLookup[inBuffer[0]>>2];
		outBuffer[1]=encodeLookup[(inBuffer[0]<<4 & 0x30) | inBuffer[1]>>4];
		outBuffer[2]=encodeLookup[((inBuffer[1] << 2) & 0x3c) | (inBuffer[2] >> 6)];
		outBuffer[3]=encodeLookup[inBuffer[2] & 0x3F];
		inBuffer = inBuffer + 3;
		outBuffer = outBuffer + 4;
	}
	if(inLengthDiv != inLength)
	{
		switch(inLength-inLengthDiv)
		{
			case 1:
				outBuffer[0]=encodeLookup[inBuffer[0]>>2];
				outBuffer[1]=encodeLookup[(inBuffer[0]<<4 & 0x30)];
				outBuffer[2]=outBuffer[3]='=';
				break;
			case 2:
				outBuffer[0]=encodeLookup[inBuffer[0]>>2];
				outBuffer[1]=encodeLookup[(inBuffer[0]<<4 & 0x30) | inBuffer[1]>>4];
				outBuffer[2]=encodeLookup[(inBuffer[1] << 2) & 0x3c];
				outBuffer[3]='=';
				break;
		}
		outBuffer = outBuffer + 4;
	}
	return [NSString stringWithCString:output length:outBuffer-output];
}

- (NSData *)initWithBase64String:(NSString *)aString
{
	NSUInteger inLength = CFStringGetLength((CFStringRef)aString);
	const unsigned char *input = (const unsigned char *)[aString cStringUsingEncoding:NSASCIIStringEncoding];
	NSUInteger outLength = inLength*3/4;
	char output[outLength];
	char encodeLookup[64] = {
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
		'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
		'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
		'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
		'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
		'w', 'x', 'y', 'z', '0', '1', '2', '3',
		'4', '5', '6', '7', '8', '9', '+', '/'
	};
	char decodeLookup[128];
	memset(decodeLookup, 0, 128*sizeof(char));
	int i;
	for(i=0; i<64; i++)
	{
		decodeLookup[encodeLookup[i]] = i;
	}
	
	char *outBuffer=output;
	const unsigned char *inBuffer=input;
	int inLengthDiv = inLength - 4;
	char bytes[4];
	while(inBuffer-input < inLengthDiv)
	{
		bytes[0]=decodeLookup[inBuffer[0]];
		bytes[1]=decodeLookup[inBuffer[1]];
		bytes[2]=decodeLookup[inBuffer[2]];
		bytes[3]=decodeLookup[inBuffer[3]];
		outBuffer[0] = bytes[0]<<2 | bytes[1]>>4;
		outBuffer[1] = bytes[1]<<4 | bytes[2]>>2;
		outBuffer[2] = bytes[2]<<6 | bytes[3];

		outBuffer += 3;
		inBuffer += 4;
	}
	bytes[0]=decodeLookup[inBuffer[0]];
	bytes[1]=decodeLookup[inBuffer[1]];
	bytes[2]=decodeLookup[inBuffer[2]];
	bytes[3]=decodeLookup[inBuffer[3]];
	outBuffer[0] = bytes[0]<<2 | bytes[1]>>4;
	if(inBuffer[2] != '=')
	{
		outBuffer[1] = bytes[1]<<4 | bytes[2]>>2;
		if(inBuffer[3] != '=')
		{
			outBuffer[2] = bytes[2]<<6 | bytes[3];
			outBuffer++;
		}
		outBuffer++;
	}
	outBuffer++;
	
	self = [self initWithBytes:output length:outBuffer-output];
	return self;
}
@end
